"use strict";
const { memsourceLib, memsourceAPI } = require("../../../memsource");
const Storage = require("../../../storage");
const utils = require("../../utils");
const config = require("../../config.json");
const PromisePool = require("es6-promise-pool");
const mapper = require("../../../mapper");
const promiseRetry = require("promise-retry");
const pantheonAPI = require("../../../pantheon-api/lib");

function pollNewProjects() {
  let authenticationToken;
  return utils
    .getAuthenticationToken(config.userMemSource, config.passMemSource)
    .then(token => (authenticationToken = token))
    .then(() => memsourceLib.getAllNewProjectsInFinancialApprovalWorkflowStep(authenticationToken))
    .then(projects => {
      const jobs = [];
      const generatePromises = function*() {
        for (let index = 0; index < projects.length; index++) {
          const projectUid = projects[index].uid;
          yield promiseRetry(
            (retry, number) => memsourceLib.getProjectWithJobs(authenticationToken, projectUid).catch(retry),
            { minTimeout: config.minTimeout }
          ).then(project => {
            project.jobs.forEach(job => {
              job.ProjectUid = project.uid;
            });

            project.processStatus = "DOWNLOADING";

            return Storage.existsProject(projectUid).then(exists => {
              if (!exists) return processProject(authenticationToken, project);

              return Promise.resolve();
            });
          });
        }
      };
      const promiseIterator = generatePromises();
      const pool = new PromisePool(promiseIterator, config.defaultConcurrentPromises);
      return Storage.initConnection()
        .then(() => pool.start())
        .then(() => Storage.closeConnection());
    })
    .catch(err => {
      console.log(err);
    });
}

function processProject(authenticationToken, project) {
  return Storage.saveProject(project)
    .then(() => {
      const promiseSaveJobs = Storage.saveJobs(project.jobs);
      let promiseCreateAnalyse;
      if (project.jobs.length !== 0 && config.createAnalyse) {
        console.log(`Creating analysis for project ${project.uid}`);
        // Asynchronous
        promiseCreateAnalyse = promiseRetry(
          (retry, number) => memsourceLib.createAnalyse(authenticationToken, project.uid, project.jobs).catch(retry),
          { minTimeout: config.minTimeout }
        );
      } else {
        promiseCreateAnalyse = Promise.resolve();
      }
      return Promise.all([Promise.resolve(project), promiseSaveJobs, promiseCreateAnalyse]);
    })
    .then(([project, jobs, asyncRequestResponse]) => {
      if (asyncRequestResponse) {
        const analyseAsyncRequestId = asyncRequestResponse.asyncRequest.id;
        const analyseId = asyncRequestResponse.analyse.id;
        const analysisRequestedDate = asyncRequestResponse.asyncRequest.dateCreated;
        return Storage.updateProject(project.uid, {
          processStatus: "ANALYSING",
          analyseId,
          analyseAsyncRequestId,
          analysisRequestedDate,
        });
      }
      Project.resolve();
    });
}

function getProjects() {
  return Storage.initConnection()
    .then(() => Storage.getAllProjectsWithJobsAndAnalyses())
    .then(projects => {
      Storage.closeConnection();
      return projects;
    })
    .catch(() => {
      Storage.closeConnection();
    });
}

function mapProject(project) {
  let dbProject;
  return Storage.initConnection()
    .then(() => Storage.getProjectWithJobsAndAnalyses(project.id))
    .then(aProject => {
      dbProject = aProject;
      return Storage.updateProjectProcessStatus(dbProject.uid, "PROCESSING");
    })
    .then(() => mapper.formatProjectData(dbProject))
    .then(formatProjectsData => pantheonAPI.submitOrderToPantheon(formatProjectsData))
    .then(() => Storage.updateProjectProcessStatus(dbProject.uid, "DONE"))
    .then(res => {
      console.log("End");
    })
    .catch(() => {
      Storage.closeConnection();
    });
}

function requestAnalysis(authenticationToken, project, jobs) {
  console.log(`Creating analysis for project ${project.uid}`);
  // Asynchronous
  return promiseRetry(
    (retry, number) => memsourceLib.createAnalyse(authenticationToken, project.uid, jobs).catch(retry),
    { minTimeout: config.minTimeout }
  ).then(asyncRequestResponse => {
    console.log(asyncRequestResponse);
    if (asyncRequestResponse) {
      const analyseAsyncRequestId = asyncRequestResponse.asyncRequest.id;
      const analyseId = asyncRequestResponse.analyse.id;
      return Storage.updateProject(project.uid, {
        processStatus: "ANALYSING",
        analyseId,
        analyseAsyncRequestId,
      });
    }
  });
}

module.exports = {
  pollNewProjects,
  processProject,
  getProjects,
  mapProject,
  requestAnalysis,
};
